
int NUM_STEPS = 1000;
float DAMPEN = 15f;
float MASS = 25f;
float STIFFNESS= 60f;
float STRETCH = 0f;
float TIME_STEP = .1f;

float RK_springInfo[][] = new float[NUM_STEPS][2];
float ES_springInfo[][] = new float[NUM_STEPS][2];
float VI_springInfo[][] = new float[NUM_STEPS][2];
ODESpring odeRK;
ODESpring odeES;
ODESpring odeVI;
int idx = 0;

boolean useSpringDraw = true;
boolean ready = false;

void setup()
{
  size(400,400,P2D);
  frame.setResizable(true);
  PFont f = loadFont("SansSerif-16.vlw");
  textFont(f, 16);
}

void startSpring()
{
  idx = 0;

  RK_springInfo = new float[NUM_STEPS][2];
  ES_springInfo = new float[NUM_STEPS][2];
  VI_springInfo = new float[NUM_STEPS][2];

  odeRK = new ODESpring(MASS, DAMPEN, STIFFNESS, STRETCH);
  odeES = new ODESpring(MASS, DAMPEN, STIFFNESS, STRETCH);
  odeVI = new ODESpring(MASS, DAMPEN, STIFFNESS, STRETCH);

  float dt = TIME_STEP;

  for (int i = 0; i < NUM_STEPS; i += 1)
  {
    odeRK.updatePositionAndVelocity_RK(dt);
    RK_springInfo[i] = new float[]{
      odeRK.q[0], odeRK.q[1]                };
    odeRK.time += dt;
    //   println("RK time: " + odeRK.getTime() + ", pos/vel = " + odeRK.q[0] + "/" + odeRK.q[1]);   

    odeES.updatePositionAndVelocity_ES(dt);
    ES_springInfo[i] = new float[]{
      odeES.q[0], odeES.q[1]                };
    odeES.time += dt;
    //  println("ES time: " + odeES.getTime() + ", pos/vel = " + odeES.q[0] + "/" + odeES.q[1]);   

    odeVI.updatePositionAndVelocity_VI(dt);
    VI_springInfo[i] = new float[]{
      odeVI.q[0], odeVI.q[1]                };
    odeVI.time += dt;
    //  println("VI time: " + odeVI.getTime() + ", pos/vel = " + odeVI.q[0] + "/" + odeVI.q[1]);   
  }
}

void draw()
{
  background(0); //clear screen

  fill(255);
  text("Dampening: " + DAMPEN + ", Stiffness: " + STIFFNESS + ", MASS: " + MASS, 3, height - 16);

  if (ready == true)
  {
    if (idx < NUM_STEPS) //draw current position
    {
      drawSprings(idx);
      idx++;  
    } 
    else //draw final position
    {
      drawSprings(NUM_STEPS - 1);
    }
  }
  else //draw mouse position
  {
    RK_springInfo[0][1] = STRETCH;
    ES_springInfo[0][1] = STRETCH;  
    VI_springInfo[0][1] = STRETCH;  
    drawSprings(0);
  }
}

//draw time vs position graph
void drawGraph(int idx)
{
  if (idx > 0)
    {
      float prevY_RK = height/2 - RK_springInfo[0][1];
      float prevY_ES = height/2 - ES_springInfo[0][1];
      float prevY_VI = height/2 - VI_springInfo[0][1];
     
      for (int i = 1; i < idx; i++)
      {
        stroke(255,0,0);
        line(i - 1, prevY_RK, i, height/2 - RK_springInfo[i][1]);
        prevY_RK = height/2 - RK_springInfo[i][1];
  
        stroke(0,0,255);
        line(i - 1, prevY_ES, i, height/2 - ES_springInfo[i][1]);
        prevY_ES = height/2 - ES_springInfo[i][1];
  
        stroke(0,255,0);
        line(i - 1, prevY_VI, i, height/2 - VI_springInfo[i][1]);
        prevY_VI = height/2 - VI_springInfo[i][1];
      }
    }
}

void drawSprings(int idx)
{
  if(useSpringDraw == true)
  {
    drawSpringees(idx);
  }
  else
  {
    drawGraph(idx);
  }
}

//draw coil from top of screen to mass at end of spring
void drawCoil(int numCoils, int xpos, int ypos, float w, float h)
{
  float hdiv = h/(numCoils - 1);
  
  //first and last are halves
  line(xpos, 0, xpos + w/2, hdiv/2);
  
  for (int i = 1; i < numCoils - 1; i+=2)
  {
    line(xpos + w/2, hdiv * i - hdiv/2, xpos-w/2, hdiv * (i +1) - hdiv/2);
    line(xpos - w/2, hdiv * (i + 1) - hdiv/2, xpos+w/2, hdiv * (i +2) - hdiv/2);
  }
  
  line (xpos + w/2, hdiv * (numCoils - 1) - hdiv/2, xpos, hdiv * numCoils - hdiv);
}

//draw the actual spring
void drawSpringees(int idx)
{
  stroke(255,0,0);
  
  drawCoil(20, width/4 * 1,0, 10, height/2 - RK_springInfo[idx][1]);
  fill(255,0,0,128);
  rect(width/4 * 1 - MASS/2, height/2 - RK_springInfo[idx][1] - MASS/2, MASS, MASS);
  fill(255);
  text("RK", width/4 * 1 + MASS/2, height/2 - RK_springInfo[idx][1] + MASS/2);
  
  stroke(0,0,255);
  drawCoil(20, width/4 * 2, 0, 10, height/2 - ES_springInfo[idx][1]); 
  fill(0,0,255,128);
  rect(width/4 * 2 - MASS/2, height/2 - ES_springInfo[idx][1] - MASS/2, MASS, MASS);
  fill(255);
  text("ES", width/4 * 2 + MASS/2, height/2 - ES_springInfo[idx][1] + MASS/2);

  stroke(0,255,0);
  drawCoil(20, width/4 * 3, 0, 10, height/2 - VI_springInfo[idx][1]); 
  fill(0,255,0, 128);
  rect(width/4 * 3 - MASS/2, height/2 - VI_springInfo[idx][1] - MASS/2, MASS, MASS);
  fill(255);
  text("VI", width/4 * 3 + MASS/2, height/2 - VI_springInfo[idx][1] + MASS/2);
}

void keyPressed()
{
  if (key == 'a')
  {
    STIFFNESS += 1;
  }
  if (key == 'z')
  {
    STIFFNESS -= 1;
  }
  if (key == 's')
  {
    DAMPEN += 1;
  }
  if (key == 'x')
  {
    DAMPEN -= 1;
  }

  if (key == ' ')
  {
    useSpringDraw = !useSpringDraw;
  }  
}

void mousePressed()
{
  if (ready == true)
  {
    ready = false;
  }
  STRETCH = height/2 - mouseY;
}

void mouseReleased()
{
  startSpring();
  ready = true;
}

void mouseDragged()
{
  STRETCH = height/2 - mouseY;
  MASS = mouseX / 5;
}


